@@ -137,6 +137,55 @@ class NumerousApp():
 
         return self.result
 
+    def _run(self):
+        """
+        Runs the simulation job with the model located in model_folder_name. Handles all events.
+        """
+        log_level = os.getenv('LOG_LEVEL', logging.root.level)
+        logger = logging.getLogger('numerous-image-tools')
+        logger.setLevel(level=log_level)
+        terminate = Event()
+
+        def handler(sig, frame):
+            logger.debug(f'main thread received {sig} {frame}')
+            terminate.set()
+
+        signal.signal(signal.SIGINT, handler)
+        signal.signal(signal.SIGTERM, handler)
+
+        try:
+
+            if self.numerous_job is None:
+                raise KeyError("no simulation job specified")
+            if self.appname is None:
+                logger.warning("no appname specified")
+
+            logger.info(f"Welcome to numerous image tools. Base version {VERSION} - app {self.appname}")
+
+            #  Create a temporary working folder, that can be used to place generated and downloaded files.
+            os.makedirs(self.working_folder, exist_ok=True)
+
+            self.terminate = terminate
+            self.client.terminate_event = terminate
+            self.loop()
+
+            self.client.set_scenario_progress(self.numerous_job._exit_code.description,
+                                             self.numerous_job._exit_code.scenariostatus,
+                                             force=True)
+
+            logger.debug(f'setting state {self.numerous_job._exit_code}')
+            self.client.state.set('_status', self.numerous_job._exit_code.name)
+
+        except Exception as e:
+            logger.error(f"unhandled exception: {tb_str(e)}")
+            self.client.set_scenario_progress("unhandled error", ScenarioStatus.FAILED, 0.0, force=True)
+            logger.debug(f'setting state -2')
+            self.client.state.set('_status', -2)
+        finally:
+            self.output.close()
+            self.client.close()
+            return self.result
+
 
 class DefaultOutputHandler:
     def __init__(self, client: NumerousClient):
@@ -159,66 +208,15 @@ def checkpoint(tnow, tlast, interval=60):
 def run_job(numerous_job=None, appname=None, max_restarts=0, model_folder="models", working_folder="tmp",
             reset_job=None):
     """
-    Runs the simulation job with the model located in model_folder_name. Handles all events.
+    Instantiates the application and runs it inside the try/except
     """
-    log_level = os.getenv('LOG_LEVEL', logging.root.level)
-    logger = logging.getLogger('numerous-image-tools')
-    logger.setLevel(level=log_level)
-    terminate = Event()
 
-    def handler(sig, frame):
-        logger.debug(f'main thread received {sig} {frame}')
-        terminate.set()
+    app = NumerousApp(appname=appname,
+                      max_restarts=max_restarts, numerous_job=numerous_job, model_folder=model_folder,
+                      working_folder=working_folder, reset_job=reset_job)
+
+    app._run()
+
 
-    signal.signal(signal.SIGINT, handler)
-    signal.signal(signal.SIGTERM, handler)
-
-    app = None
-
-    try:
-        if reset_job is None:
-            RESET_JOB = os.getenv('RESET_JOB', 'False') == "True"
-        else:
-            RESET_JOB = reset_job
-
-        if numerous_job is None:
-            raise KeyError("no simulation job specified")
-        if appname is None:
-            logger.warning("no appname specified")
-
-        logger.info(f"Welcome to numerous image tools. Base version {VERSION} - app {appname}")
-
-        #  Create a temporary working folder, that can be used to place generated and downloaded files.
-        os.makedirs(working_folder, exist_ok=True)
-
-        app = NumerousApp(appname=appname,
-                          max_restarts=max_restarts, numerous_job=numerous_job, model_folder=model_folder,
-                          working_folder=working_folder, reset_job=RESET_JOB)
-
-        app.terminate = terminate
-        app.client.terminate_event = terminate
-        app.loop()
-
-        app.client.set_scenario_progress(numerous_job._exit_code.description, numerous_job._exit_code.scenariostatus,
-                                 force=True)
-
-        logger.debug(f'setting state {numerous_job._exit_code}')
-        app.client.state.set('_status', numerous_job._exit_code.name)
-
-    except Exception as e:
-        logger.error(f"unhandled exception: {tb_str(e)}")
-        if app and app.client is not None:
-            app.client.set_scenario_progress("unhandled error", ScenarioStatus.FAILED, 0.0, force=True)
-            logger.debug(f'setting state -2')
-            app.client.state.set('_status', -2)
-    finally:
-        if app and app.client is not None:
-            app.output.close()
-            app.client.close()
-        if app:
-            #Return the result of the app
-            return app.result
-
-    return
 
 
