@@ -54,6 +54,28 @@ class NumerousBaseJob:
         # Wrapper to allow specialized instanciation of components
         self.wrapper = None
 
+    def read_data(self, t):
+
+        if self.app.client.terminate_event.is_set():
+            raise TerminateJob
+        if self.app.client.hibernate_event.is_set():
+            raise HibernateJob
+
+        while True:
+            try:
+                data = self.input.get_at_time(t)
+            except DataSourceStreamStarting:
+                self.app.client.set_scenario_progress('waiting for input stream to start',
+                                                      ScenarioStatus.INITIALZING, 0.0)
+                sleep(0.1)
+                continue
+            except Exception:
+                raise
+
+            self.system.update_inputs(data)
+
+            return data
+
     def _post_init(self):  # This is called before _run_job from the app.loop method
         timeout = 120
         t, states = self._load_states()
@@ -177,30 +199,45 @@ class HibernateJob(Exception):
 class NumerousSimulationJob(NumerousBaseJob):
     def __init__(self):
         super(NumerousSimulationJob, self).__init__()
-        self.system = None
+        self.system: Optional[NumerousSystem] = None
         #Specifies whether or not a job will save its results
         self.align_outputs_to_next_timestep = True
         self.logger = logging.getLogger('numerous-simulation-job')
         self.logger.setLevel(level=LOG_LEVEL)
+        self._completed = 0
+        self._t_last = None
 
-    def read_data(self, t):
+    def write_output(self, t, output):
+        if not output:
+            raise SimulationError("Expected an output, but got None")
 
-        if self.app.client.terminate_event.is_set():
-            raise TerminateJob
-        if self.app.client.hibernate_event.is_set():
-            raise HibernateJob
+        if self.init:
+            self.app.client.set_timeseries_meta_data([{"name": tag} for tag in output.keys()],
+                                                     offset=self.app.start_time)
+            self.init = False
 
-        while True:
-            try:
-                data = self.input.get_at_time(t)
-            except DataSourceStreamStarting:
-                self.app.client.set_scenario_progress('waiting for input stream to start',
-                                                      ScenarioStatus.INITIALZING, 0.0)
-                sleep(0.1)
-                continue
-            except Exception:
-                raise
-            return data
+        if not self.align_outputs_to_next_timestep:
+            output.update({'_index': self._t - self.app.start_time})
+        else:
+            output.update({'_index': t - self.app.start_time})
+
+        self.app.output.write_row(t, output)
+
+    def read_input(self, t: float) -> (dict, float):
+        data = self.read_data(t)
+        if not data:
+            self.app.client.set_scenario_progress(f"waiting for data. Last update: "
+                                                  f"{datetime.fromtimestamp(t)}",
+                                                  ScenarioStatus.RUNNING if not self.init else ScenarioStatus.WAITING,
+                                                  self._completed)
+            if self._print_update(datetime.now().timestamp(), 0, 10):
+                self.logger.info(f"no data. Simulation time: {t}. ")
+                sleep(1)
+            return self.read_input(t)
+
+        if t == 0 and data['_index'] > 0:
+            self.logger.warning(f'setting start time to {data["_index"]}')
+            self._t = data['_index']
 
     def initialize_simulation_system(self) -> Optional[dict]:
         """
@@ -246,8 +283,8 @@ class NumerousSimulationJob(NumerousBaseJob):
 
         details = ""
         i = 0
-        last_data = None
-        data = None
+        last_data = {}
+
         t = None
         exit_code = ExitCode.COMPLETED
         try:
@@ -256,7 +293,7 @@ class NumerousSimulationJob(NumerousBaseJob):
             self.app.client.set_scenario_progress('waiting for initial data', ScenarioStatus.INITIALZING, 0.0, force=True)
 
             t_stop = self.system.end_time
-            t = self._t0
+            self._t = self._t0
 
             #Increasingly longer spacing between log updates to 60 s
             last_logged = -1000
@@ -264,79 +301,47 @@ class NumerousSimulationJob(NumerousBaseJob):
             dt_log = 1
 
             while True:
-                data = self.read_data(t)
 
                 completed = 0
                 if t_stop > 0:
-                    completed = (1 - (t_stop - t) / (t_stop - self.app.start_time)) * 100
+                    self._completed = (1 - (t_stop - self._t) / (t_stop - self.app.start_time)) * 100
+                    self.app.client.set_scenario_progress("running", ScenarioStatus.RUNNING, completed)
 
                 i+=1
                 if self.app.backup.is_set():
                     self._save_states(t, 'backup', f"scheduled checkpoint @ {datetime.now()}")
                     self.app.backup.clear()
 
-                if data is None:
-                    self.app.client.set_scenario_progress(f"waiting for data. Last update: "
-                                                  f"{datetime.fromtimestamp(t)}",
-                                                  ScenarioStatus.RUNNING if not self.init else ScenarioStatus.WAITING,
-                                                  completed)
-                    if self._print_update(datetime.now().timestamp(), 0, 10):
-                        self.logger.info(f"no data. Simulation time: {t}. ")
-                        sleep(1)
-                    continue
-
-                self.system.update_inputs(data)
-
-                if t_stop > 0:
-                    self.app.client.set_scenario_progress("running", ScenarioStatus.RUNNING, completed)
-
-                if t==0 and data['_index'] > 0:
-                    self.logger.warning(f'setting start time to {data["_index"]}')
-                    t = data['_index']
-
-                # if data is a dict, then convert to list
+                self.read_input(self._t)
 
                 if self.init:
-                    self.app.client.set_scenario_progress("building model", ScenarioStatus.MODEL_INITIALIZING, 0, force=True)
+                    self.app.client.set_scenario_progress("building model", ScenarioStatus.MODEL_INITIALIZING, 0,
+                                                          force=True)
                     initial_output = self.initialize_simulation_system()
                     if initial_output:
                         if "_index" not in initial_output:
                             initial_output.update({'_index': 0})
-
-                        self.app.output.write_row(t, initial_output)
+                        self.write_output(self._t, initial_output)
 
                 try:
-                    tnew, outputs = self.step(t, self.system.dt)
-                    if not outputs:
-                        continue
-                    if self.init:
-                        self.app.client.set_timeseries_meta_data([{"name": tag} for tag in outputs.keys()],
-                                                         offset=self.app.start_time)
-                        self.init = False
-
-                    if not self.align_outputs_to_next_timestep:
-                        outputs.update({'_index': t - self.app.start_time})
-                    else:
-                        outputs.update({'_index': tnew - self.app.start_time})
-
+                    tnew, output = self.step(self._t, self.system.dt)
+                    self.write_output(tnew, output)
                 except Exception as e:
                     details = tb_str(e)
                     raise SimulationError(details)
 
                 # Advance time
-                t = tnew
-                # save output
-                self.app.output.write_row(t, outputs)
+                self._t = tnew
 
                 if time() - last_logged > dt_log:
                     last_logged = time()
                     dt_log = min(dt_log_max, dt_log*2)
 
-                    self.logger.info(f'Calculation step. Time is now {t}. completed: {completed}')
+                    self.logger.info(f'Calculation step. Time is now {self._t}. completed: {self._completed}')
 
-                last_data = data
+                last_data = {component.name: component.inputs for component in self.system.components.values()}
 
-                if (t >= t_stop) and (t_stop > 0):
+                if (self._t >= t_stop) and (t_stop > 0):
                     self.logger.warning('maximum time reached')
                     exit_code = ExitCode.COMPLETED
                     break
@@ -349,11 +354,11 @@ class NumerousSimulationJob(NumerousBaseJob):
         except Exception as e:
             self.logger.error(f'numerous_app crashed: {tb_str(e)}')
             self.logger.debug(f'previous data: {last_data}')
+            data = {component.name: component.inputs for component in self.system.components.values()}
             self.logger.debug(f'data at crash: {data}')
             details = {"error message": tb_str(e), "previous_data": last_data, "data at crash": data}
             exit_code = ExitCode.ERROR
         finally:
             self.logger.warning('job terminated')
-            self._t = t
             self._details = details
             return exit_code
\ No newline at end of file
