@@ -16,9 +16,10 @@ from typing import List
 logger = logging.getLogger('sim-setup')
 
 class Component:
-    def __init__(self, component_id:str, container_id: str, name: str = None, folder: str = None, path: str = None, handle: str = None,
+    def __init__(self, component_id:str, container_id: str, name: str = None, folder: str = None, path: str = None,
+                 handle: str = None,
                  constants: dict = None, parameters: dict = None,
-                 import_flag = False):
+                 import_flag = False, item_class = None):
 
         self._folder = Path(folder)
         self._path = Path(path)
@@ -28,6 +29,7 @@ class Component:
         self._constructor = None
         self._entrypoint = None
         self.item = None
+        self.item_class = item_class
         self.name = name
         self.component_id = component_id
         self.container_id = container_id
@@ -270,6 +272,8 @@ class NumerousSystem:
         if subcomps:
             subcomponents_list = self._find_subcomponents(subcomps, simcomponents)
             for subcomponent in subcomponents_list:
+                if subcomponent.get('disabled', False):
+                    continue
                 subcomponent = self._add_components_recursive(simcomponents, subcomponent, level=level+1)
                 component._add_subcomponent(subcomponent)
         return component
@@ -286,7 +290,7 @@ class NumerousSystem:
         item_class = simcomponent['item_class']
         items = item_class.split(".")
         component_name = simcomponent['name']
-        component_id = simcomponent['id']
+        component_id = simcomponent['uuid']
         container_id = simcomponent['containerID'] if 'containerID' in simcomponent else None
 
         if item_class and len(items) > 1:
@@ -337,6 +341,7 @@ class NumerousSystem:
                               folder=folder,
                               path=path, handle=handle,
                               constants=component_constants,
-                              parameters=component_parameters, import_flag=has_model and self._model_folder is not None)
+                              parameters=component_parameters, import_flag=has_model and self._model_folder is not None,
+                              item_class=item_class)
 
         return component
